@@ -10,6 +10,7 @@ use Data::Dumper;
 use Storable qw(dclone);
 use feature 'switch';
 use Gscan2pdf::Scanner::Options;
+use Gscan2pdf::Scanner::Profile;
 my (
     $_MAX_PAGES,        $_MAX_INCREMENT, $_DOUBLE_INCREMENT,
     $_CANVAS_SIZE,      $_CANVAS_BORDER, $_CANVAS_POINT_SIZE,
@@ -204,11 +205,12 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
         'Gscan2pdf::Scanner::Options',       # package
         [qw/readable writable/]              # flags
     ),
-    Glib::ParamSpec->scalar(
-        'current-scan-options',                               # name
-        'Current scan options',                               # nick
-        'Array of scan options making up current profile',    # blurb
-        [qw/readable/]                                        # flags
+    Glib::ParamSpec->object(
+        'current-scan-options',                      # name
+        'Current scan options',                      # nickname
+        'Scan options making up current profile',    # blurb
+        'Gscan2pdf::Scanner::Profile',               # package
+        [qw/readable/]                               # flags
     ),
     Glib::ParamSpec->scalar(
         'visible-scan-options',                                  # name
@@ -234,7 +236,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
     ),
   ];
 
-our $VERSION = '1.5.5';
+our $VERSION = '1.6.0';
 
 my ( $d, $d_sane, $logger, $tooltips );
 my $tolerance             = 1;
@@ -489,6 +491,7 @@ sub INIT_INSTANCE {
     );
 
     # Scan profiles
+    $self->{current_scan_options} = Gscan2pdf::Scanner::Profile->new;
     my $framesp = Gtk2::Frame->new( $d->get('Scan profiles') );
     $vbox1->pack_start( $framesp, FALSE, FALSE, 0 );
     my $vboxsp = Gtk2::VBox->new;
@@ -739,7 +742,8 @@ sub SET_PROPERTY {
                     }
                 );
                 $self->set_paper($newval);
-                $self->{current_scan_options}{frontend}{$name} = $newval;
+                $self->{current_scan_options}
+                  ->add_frontend_option( $name, $newval );
             }
             when ('paper_formats') {
                 $self->{$name} = $newval;
@@ -826,7 +830,7 @@ sub _set_num_pages {
       )
     {
         $self->{$name} = $newval;
-        $self->{current_scan_options}{frontend}{$name} = $newval;
+        $self->{current_scan_options}->add_frontend_option( $name, $newval );
         $self->signal_emit( 'changed-num-pages', $newval );
     }
     return;
@@ -1096,11 +1100,18 @@ sub update_options {
     my $num_dev_options = $new_options->num_options;
     my $options         = $self->get('available-scan-options');
     for ( 1 .. $num_dev_options - 1 ) {
-        my $opt    = $options->by_index($_);
-        my $widget = $self->{option_widgets}{ $opt->{name} };
+        my $opt = $options->by_index($_);
 
         # could be undefined for !($new_opt->{cap} & SANE_CAP_SOFT_DETECT)
-        if ( not defined $widget ) { next }
+        # or where $opt->{name} is not defined
+        # e.g. $opt->{type} == SANE_TYPE_GROUP
+        if (   $opt->{type} == SANE_TYPE_GROUP
+            or not defined $opt->{name}
+            or not defined $self->{option_widgets}{ $opt->{name} } )
+        {
+            next;
+        }
+        my $widget = $self->{option_widgets}{ $opt->{name} };
 
         my $new_opt = $new_options->by_index($_);
         if ( $new_opt->{name} ne $opt->{name} ) {
@@ -1115,7 +1126,6 @@ sub update_options {
             );
             return;
         }
-        if ( $opt->{type} == SANE_TYPE_GROUP ) { next }
 
         # Block the signal handler for the widget to prevent infinite
         # loops of the widget updating the option, updating the widget, etc.
@@ -1248,10 +1258,9 @@ sub set_paper {
             return;
         }
     }
-    my $formats = $self->get('paper-formats');
-    my $options = $self->get('available-scan-options');
-    my $paper_profile;
-    $paper_profile->{backend} = [];
+    my $formats       = $self->get('paper-formats');
+    my $options       = $self->get('available-scan-options');
+    my $paper_profile = Gscan2pdf::Scanner::Profile->new;
     if ( defined( $options->by_name(SANE_NAME_PAGE_HEIGHT) )
         and not $options->by_name(SANE_NAME_PAGE_HEIGHT)->{cap} &
         SANE_CAP_INACTIVE
@@ -1259,39 +1268,33 @@ sub set_paper {
         and not $options->by_name(SANE_NAME_PAGE_WIDTH)->{cap} &
         SANE_CAP_INACTIVE )
     {
-        $self->build_profile(
-            $paper_profile,
-            $options->by_name(SANE_NAME_PAGE_HEIGHT),
-            $formats->{$paper}{y} + $formats->{$paper}{t}
+        $paper_profile->add_backend_option( SANE_NAME_PAGE_HEIGHT,
+            $formats->{$paper}{y} + $formats->{$paper}{t},
+            $options->by_name(SANE_NAME_PAGE_HEIGHT)->{val}
         );
-        $self->build_profile(
-            $paper_profile,
-            $options->by_name(SANE_NAME_PAGE_WIDTH),
-            $formats->{$paper}{x} + $formats->{$paper}{l}
+        $paper_profile->add_backend_option( SANE_NAME_PAGE_WIDTH,
+            $formats->{$paper}{x} + $formats->{$paper}{l},
+            $options->by_name(SANE_NAME_PAGE_WIDTH)->{val}
         );
     }
-    $self->build_profile(
-        $paper_profile,
-        $options->by_name(SANE_NAME_SCAN_TL_X),
-        $formats->{$paper}{l}
+    $paper_profile->add_backend_option( SANE_NAME_SCAN_TL_X,
+        $formats->{$paper}{l},
+        $options->by_name(SANE_NAME_SCAN_TL_X)->{val}
     );
-    $self->build_profile(
-        $paper_profile,
-        $options->by_name(SANE_NAME_SCAN_TL_Y),
-        $formats->{$paper}{t}
+    $paper_profile->add_backend_option( SANE_NAME_SCAN_TL_Y,
+        $formats->{$paper}{t},
+        $options->by_name(SANE_NAME_SCAN_TL_Y)->{val}
     );
-    $self->build_profile(
-        $paper_profile,
-        $options->by_name(SANE_NAME_SCAN_BR_X),
-        $formats->{$paper}{x} + $formats->{$paper}{l}
+    $paper_profile->add_backend_option( SANE_NAME_SCAN_BR_X,
+        $formats->{$paper}{x} + $formats->{$paper}{l},
+        $options->by_name(SANE_NAME_SCAN_BR_X)->{val}
     );
-    $self->build_profile(
-        $paper_profile,
-        $options->by_name(SANE_NAME_SCAN_BR_Y),
-        $formats->{$paper}{y} + $formats->{$paper}{t}
+    $paper_profile->add_backend_option( SANE_NAME_SCAN_BR_Y,
+        $formats->{$paper}{y} + $formats->{$paper}{t},
+        $options->by_name(SANE_NAME_SCAN_BR_Y)->{val}
     );
 
-    if ( not @{ $paper_profile->{backend} } ) {
+    if ( not $paper_profile->num_backend_options ) {
         $self->hide_geometry($options);
         $self->{paper} = $paper;
         $self->signal_emit( 'changed-paper', $paper );
@@ -1309,9 +1312,11 @@ sub set_paper {
         }
     );
 
-# Don't trigger the changed-paper signal until we have finished setting the profile
+    # Don't trigger the changed-paper signal
+    # until we have finished setting the profile
     $self->{setting_profile} = TRUE;
-    $self->_set_option_profile( 0, $paper_profile );
+    $self->_set_option_profile( $paper_profile,
+        $paper_profile->each_backend_option );
     return;
 }
 
@@ -1504,19 +1509,30 @@ sub save_current_profile {
 
 sub add_profile {
     my ( $self, $name, $profile ) = @_;
-    if ( defined $name and defined $profile ) {
-
-        # if we don't clone the profile,
-        # we get strange action-at-a-distance problems
-        $self->{profiles}{$name} = dclone($profile);
-
-        _combobox_remove_item_by_text( $self->{combobsp}, $name );
-
-        $self->{combobsp}->append_text($name);
-        $logger->debug( "Saved profile '$name':",
-            Dumper( $self->{profiles}{$name} ) );
-        $self->signal_emit( 'added-profile', $name, $self->{profiles}{$name} );
+    if ( not defined $name ) {
+        $logger->error('Cannot add profile with no name');
+        return;
     }
+    elsif ( not defined $profile ) {
+        $logger->error('Cannot add undefined profile');
+        return;
+    }
+    elsif ( ref($profile) ne 'Gscan2pdf::Scanner::Profile' ) {
+        $logger->error(
+            ref($profile) . ' is not a Gscan2pdf::Scanner::Profile object' );
+        return;
+    }
+
+    # if we don't clone the profile,
+    # we get strange action-at-a-distance problems
+    $self->{profiles}{$name} = dclone($profile);
+
+    _combobox_remove_item_by_text( $self->{combobsp}, $name );
+
+    $self->{combobsp}->append_text($name);
+    $logger->debug( "Saved profile '$name':",
+        Dumper( $self->{profiles}{$name}->get_data ) );
+    $self->signal_emit( 'added-profile', $name, $self->{profiles}{$name} );
     return;
 }
 
@@ -1554,21 +1570,7 @@ sub set_profile {
                 $self->signal_emit( 'changed-profile', $name );
             }
         );
-
-        # for reasons I don't understand, without walking the reference tree,
-        # parts of $self->{profiles}{$name} are undef
-        Dumper( $self->{profiles}{$name} );
-
-        # I don't understand why it necessary to clone the profile here,
-        # since it is cloned again in set_current_scan_options(),
-        # but it shouldn't hurt, and otherwise, it fails t/0601_Dialog_Scan.t
-
-        # I also don't understand why the $clone variable is necessary: why
-        # I can't simply pass the return value from dclone() directly
-        # into set_current_scan_options(),
-        # but otherwise it fails t/0610_Dialog_Scan.t
-        my $clone = dclone( $self->{profiles}{$name} );
-        $self->set_current_scan_options($clone);
+        $self->set_current_scan_options( $self->{profiles}{$name} );
     }
 
     # no need to wait - nothing to do
@@ -1593,16 +1595,6 @@ sub remove_profile {
     return;
 }
 
-# Helper function to check the new value is different before adding the option
-# to the profile
-sub build_profile {
-    my ( $self, $profile, $option, $newval ) = @_;
-    if ( $option->{val} != $newval ) {
-        push @{ $profile->{backend} }, { $option->{name} => $newval };
-    }
-    return;
-}
-
 sub get_combobox_num_rows {
     my ($combobox) = @_;
     if ( not defined $combobox ) { return }
@@ -1991,39 +1983,42 @@ sub update_graph {
 
 sub set_current_scan_options {
     my ( $self, $profile ) = @_;
+    if ( not defined $profile ) {
+        $logger->error('Cannot add undefined profile');
+        return;
+    }
+    elsif ( ref($profile) ne 'Gscan2pdf::Scanner::Profile' ) {
+        $logger->error(
+            ref($profile) . ' is not a Gscan2pdf::Scanner::Profile object' );
+        return;
+    }
 
-    if ( not defined $profile ) { return }
     $self->{setting_current_scan_options} = TRUE;
 
     # First clone the profile, as otherwise it would be self-modifying
     my $clone = dclone($profile);
 
-    # As scanimage and scanadf rename the geometry options,
-    # we have to map them back to the original names
-    $self->_map_geometry_names( $clone->{backend} );
-
     # Give the GUI a chance to catch up between settings,
     # in case they have to be reloaded.
     # Use the callback to trigger the next loop
-    $self->_set_option_profile( 0, $clone );
+    $self->_set_option_profile( $clone, $clone->each_backend_option );
     return;
 }
 
 sub _set_option_profile {
-    my ( $self, $i, $profile ) = @_;
-    if ( $i < @{ $profile->{backend} } ) {
+    my ( $self, $profile, $next, $step ) = @_;
 
-        # for reasons I don't understand, without walking the reference tree,
-        # parts of $profile->{backend} are undef
-        Dumper( $profile->{backend}[$i] );
-        my ( $name, $val ) = each %{ $profile->{backend}[$i] };
+    if ( my $i = $next->($step) ) {
+        my ( $name, $val ) = $profile->get_backend_option_by_index($i);
 
         my $options = $self->get('available-scan-options');
         my $opt     = $options->by_name($name);
         if ( not defined $opt or $opt->{cap} & SANE_CAP_INACTIVE ) {
             $logger->warn("Ignoring inactive option '$name'.");
-            splice @{ $profile->{backend} }, $i, 1;
-            $self->_set_option_profile( $i, $profile );
+            $profile->remove_backend_option_by_index($i);
+
+            # no increment, as we have removed an option
+            $self->_set_option_profile( $profile, $next, FALSE );
             return;
         }
 
@@ -2032,7 +2027,7 @@ sub _set_option_profile {
         $signal = $self->signal_connect(
             'changed-scan-option' => sub {
                 $self->signal_handler_disconnect($signal);
-                $self->_set_option_profile( $i + 1, $profile );
+                $self->_set_option_profile( $profile, $next );
             }
         );
         $self->set_option( $opt, $val );
@@ -2079,10 +2074,9 @@ sub _set_option_profile {
     else {
 
         # Having set all backend options, set the frontend options
-        if ( defined $profile->{frontend} ) {
-            for my $key ( keys %{ $profile->{frontend} } ) {
-                $self->set( $key, $profile->{frontend}{$key} );
-            }
+        my $iter = $profile->each_frontend_option;
+        while ( my $key = $iter->() ) {
+            $self->set( $key, $profile->get_frontend_option($key) );
         }
 
         if ( not $self->{setting_profile} ) {
@@ -2094,65 +2088,6 @@ sub _set_option_profile {
             $self->get('current-scan-options')
         );
     }
-    return $i;
-}
-
-# Map scanimage and scanadf geometry options back to the original names
-
-sub _map_geometry_names {
-    my ( $self, $options ) = @_;
-    for my $i ( 0 .. $#{$options} ) {
-
-        # for reasons I don't understand, without walking the reference tree,
-        # parts of $options are undef
-        Dumper($options);
-        my ( $name, $val ) = each %{ $options->[$i] };
-        given ($name) {
-            when ('l') {
-                $name = SANE_NAME_SCAN_TL_X;
-                $options->[$i] = { $name => $val };
-            }
-            when ('t') {
-                $name = SANE_NAME_SCAN_TL_Y;
-                $options->[$i] = { $name => $val };
-            }
-            when ('x') {
-                $name = SANE_NAME_SCAN_BR_X;
-                my $l = get_option_from_profile( 'l', $options );
-                if ( not defined $l ) {
-                    $l =
-                      get_option_from_profile( SANE_NAME_SCAN_TL_X, $options );
-                }
-                if ( defined $l ) { $val += $l }
-                $options->[$i] = { $name => $val };
-            }
-            when ('y') {
-                $name = SANE_NAME_SCAN_BR_Y;
-                my $t = get_option_from_profile( 't', $options );
-                if ( not defined $t ) {
-                    $t =
-                      get_option_from_profile( SANE_NAME_SCAN_TL_Y, $options );
-                }
-                if ( defined $t ) { $val += $t }
-                $options->[$i] = { $name => $val };
-            }
-        }
-    }
-    return;
-}
-
-# Extract a option value from a profile
-
-sub get_option_from_profile {
-    my ( $name, $profile ) = @_;
-
-    # for reasons I don't understand, without walking the reference tree,
-    # parts of $profile are undef
-    Dumper($profile);
-    for ( @{$profile} ) {
-        my ( $key, $val ) = each %{$_};
-        return $val if ( $key eq $name );
-    }
     return;
 }
 
@@ -2163,22 +2098,6 @@ sub make_progress_string {
     return sprintf $d->get('Scanning page %d'), $i;
 }
 
-sub add_to_current_scan_options {
-    my ( $self, $option, $val ) = @_;
-    my $current = $self->{current_scan_options};
-    if ( not defined $current->{backend} ) {
-        $current->{backend} = [];
-    }
-
-    # Cache option
-    push @{ $current->{backend} }, { $option->{name} => $val };
-
-    # Note any duplicate options, keeping only the last entry.
-    $self->{current_scan_options}{backend} =
-      Gscan2pdf::Scanner::Options::prune_duplicates( $current->{backend} );
-    return;
-}
-
 1;
 
 __END__
