@@ -79,7 +79,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::Scan::, signals => {
     ),
   ];
 
-our $VERSION = '1.5.5';
+our $VERSION = '1.6.0';
 
 my $SANE_NAME_PAGE_HEIGHT = SANE_NAME_PAGE_HEIGHT;
 my $SANE_NAME_PAGE_WIDTH  = SANE_NAME_PAGE_WIDTH;
@@ -172,7 +172,6 @@ sub cache_key {
 
     my $cache_key = $EMPTY;
     if ( defined $options ) {
-
         for my $opt ( @{ $options->{array} } ) {
             for ( @{$reload_triggers} ) {
                 if ( defined( $opt->{name} ) and /^$opt->{name}$/ixsm ) {
@@ -184,14 +183,10 @@ sub cache_key {
         }
     }
     else {
-
-        # for reasons I don't understand, without walking the reference tree,
-        # parts of $default are undef
-        Dumper( $self->{current_scan_options} );
-
-        # grep the reload triggers from the current options
-        for ( @{ $self->{current_scan_options}{backend} } ) {
-            my ( $key, $value ) = each %{$_};
+        my $next = $self->{current_scan_options}->each_backend_option;
+        while ( my $i = $next->() ) {
+            my ( $key, $value ) =
+              $self->{current_scan_options}->get_backend_option_by_index($i);
             for ( @{$reload_triggers} ) {
                 if (/^$key$/ixsm) {
                     if ( $cache_key ne $EMPTY ) { $cache_key .= $COMMA }
@@ -252,7 +247,7 @@ sub scan_options {
         prefix           => $self->get('prefix'),
         frontend         => $self->get('frontend'),
         device           => $self->get('device'),
-        options          => $self->{current_scan_options}{backend},
+        options          => $self->{current_scan_options},
         started_callback => sub {
 
             # Set up ProgressBar
@@ -589,7 +584,7 @@ sub set_option {
     $option->{val} = $val;
     $self->update_widget( $option->{name}, $val );
 
-    $self->add_to_current_scan_options( $option, $val );
+    $self->{current_scan_options}->add_backend_option( $option->{name}, $val );
 
     # Do we need to reload?
     my $reload_triggers = $self->get('reload-triggers');
@@ -640,8 +635,8 @@ sub set_option {
                 prefix   => $self->get('prefix'),
                 frontend => $self->get('frontend'),
                 device   => $self->get('device'),
-                options =>
-                  $self->map_options( $self->{current_scan_options}{backend} ),
+                options  => $self->{current_scan_options}
+                  ->map_to_cli( $self->get('available-scan-options') ),
                 started_callback => sub {
 
                     # Set up ProgressBar
@@ -727,13 +722,10 @@ sub _set_option_flags_signals {
 
 sub patch_cache {
     my ( $self, $options ) = @_;
-
-    # for reasons I don't understand, without walking the
-    # reference tree, parts of $self->{current_scan_options}
-    # are undef
-    Dumper( $self->{current_scan_options} );
-    for my $hashref ( @{ $self->{current_scan_options}{backend} } ) {
-        my ( $key, undef ) = each %{$hashref};
+    my $next = $self->{current_scan_options}->each_backend_option;
+    while ( my $i = $next->() ) {
+        my ($key) =
+          $self->{current_scan_options}->get_backend_option_by_index($i);
         my $updated_option =
           $self->get('available-scan-options')->by_name($key);
         if ( defined( $updated_option->{name} )
@@ -816,26 +808,6 @@ sub update_widget {    # FIXME: this is partly duplicated in Sane.pm
     return;
 }
 
-# change boolean values from TRUE and FALSE to yes and no
-sub map_options {
-    my ( $self, $old ) = @_;
-    my @new;
-    my $options = $self->get('available-scan-options');
-    for ( @{$old} ) {
-
-        # for reasons I don't understand, without walking the reference tree,
-        # parts of $_ are undef
-        Dumper($_);
-        my ( $key, $val ) = each %{$_};
-        my $opt = $options->by_name($key);
-        if ( defined( $opt->{type} ) and $opt->{type} == SANE_TYPE_BOOL ) {
-            $val = $val ? 'yes' : 'no';
-        }
-        push @new, { $key => $val };
-    }
-    return \@new;
-}
-
 sub scan {
     my ($self) = @_;
 
@@ -851,16 +823,13 @@ sub scan {
         return TRUE;
     }
 
-    # As scanimage and scanadf rename the geometry options,
-    # we have to map them back to the original names
-    my $options = $self->{current_scan_options}{backend};
-
     my $i = 1;
     Gscan2pdf::Frontend::CLI->scan_pages(
-        device           => $self->get('device'),
-        dir              => $self->get('dir'),
-        format           => 'out%d.pnm',
-        options          => $self->map_options($options),
+        device  => $self->get('device'),
+        dir     => $self->get('dir'),
+        format  => 'out%d.pnm',
+        options => $self->{current_scan_options}
+          ->map_to_cli( $self->get('available-scan-options') ),
         npages           => $npages,
         start            => $start,
         step             => $step,
