@@ -31,7 +31,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::Scan::, properties => [
     ),
 ];
 
-our $VERSION = '1.5.5';
+our $VERSION = '1.6.0';
 
 my $SANE_NAME_SCAN_TL_X   = SANE_NAME_SCAN_TL_X;
 my $SANE_NAME_SCAN_TL_Y   = SANE_NAME_SCAN_TL_Y;
@@ -357,7 +357,7 @@ sub set_option {
     my ( $self, $option, $val ) = @_;
     if ( not defined $option ) { return }
 
-    $self->add_to_current_scan_options( $option, $val );
+    $self->{current_scan_options}->add_backend_option( $option->{name}, $val );
 
     my $signal;
     my $options = $self->get('available-scan-options');
