@@ -17,7 +17,6 @@ use Gscan2pdf::NetPBM;
 use Gscan2pdf::Scanner::Options;
 use Gscan2pdf::Dialog::Scan;
 use Sane 0.05;    # To get SANE_NAME_PAGE_WIDTH & SANE_NAME_PAGE_HEIGHT
-use Data::Dumper;
 use Cwd;
 use File::Spec;
 use Readonly;
@@ -27,7 +26,7 @@ Readonly my $_SANE_STATUS_EOF             => 5;      # or we could use Sane
 Readonly my $_1KB                         => 1024;
 Readonly my $ALL_PENDING_ZOMBIE_PROCESSES => -1;
 
-our $VERSION = '1.5.5';
+our $VERSION = '1.6.0';
 
 my $EMPTY = q{};
 my $COMMA = q{,};
@@ -297,21 +296,18 @@ sub _create_scanimage_cmd {
     # inverted commas needed for strange characters in device name
     my $device = "--device-name='$options{device}'";
 
-    # Add basic options
-    _map_geometry_names( $options{options} );
-
-    # for reasons I don't understand, without walking the reference tree,
-    # parts of $options are undef
-    Dumper( $options{options} );
-
     my @options;
-    for ( @{ $options{options} } ) {
-        my ( $key, $value ) = each %{$_};
-        if ( $key =~ /^[xytl]$/xsm ) {
-            push @options, "-$key $value";
-        }
-        else {
-            push @options, "--$key='$value'";
+    if ( defined $options{options} ) {
+        my $iter = $options{options}->each_backend_option;
+        while ( my $i = $iter->() ) {
+            my ( $key, $val ) =
+              $options{options}->get_backend_option_by_index($i);
+            if ( $key =~ /^[xytl]$/xsm ) {
+                push @options, "-$key $val";
+            }
+            else {
+                push @options, "--$key='$val'";
+            }
         }
     }
     if ( not $help ) {
@@ -332,51 +328,6 @@ sub _create_scanimage_cmd {
     return "$options{prefix} $options{frontend} $help $device @options";
 }
 
-# As scanimage and scanadf rename the geometry options,
-# we have to map them back to the original names
-sub _map_geometry_names {
-    my ($options) = @_;
-    for my $i ( 0 .. $#{$options} ) {
-
-        # for reasons I don't understand, without walking the reference tree,
-        # parts of $options are undef
-        Dumper( $options->[$i] );
-
-        my ( $name, $val ) = each %{ $options->[$i] };
-        given ($name) {
-            when (SANE_NAME_SCAN_TL_X) {
-                $options->[$i] = { l => $val };
-            }
-            when (SANE_NAME_SCAN_TL_Y) {
-                $options->[$i] = { t => $val };
-            }
-            when (SANE_NAME_SCAN_BR_X) {
-                my $l = Gscan2pdf::Dialog::Scan::get_option_from_profile( 'l',
-                    $options );
-                if ( not defined $l ) {
-                    $l =
-                      Gscan2pdf::Dialog::Scan::get_option_from_profile( SANE_NAME_SCAN_TL_X,
-                        $options );
-                }
-                if ( defined $l ) { $val -= $l }
-                $options->[$i] = { x => $val };
-            }
-            when (SANE_NAME_SCAN_BR_Y) {
-                my $t = Gscan2pdf::Dialog::Scan::get_option_from_profile( 't',
-                    $options );
-                if ( not defined $t ) {
-                    $t =
-                      Gscan2pdf::Dialog::Scan::get_option_from_profile( SANE_NAME_SCAN_TL_Y,
-                        $options );
-                }
-                if ( defined $t ) { $val -= $t }
-                $options->[$i] = { y => $val };
-            }
-        }
-    }
-    return;
-}
-
 # Carry out the scan with scanadf and the options passed.
 
 sub _scanadf {
