@@ -32,7 +32,7 @@ BEGIN {
     use Exporter ();
     our ( $VERSION, @EXPORT_OK, %EXPORT_TAGS );
 
-    $VERSION = '1.5.5';
+    $VERSION = '1.6.0';
 
     use base qw(Exporter);
     %EXPORT_TAGS = ();    # eg: TAG => [ qw!name1 name2! ],
@@ -526,7 +526,10 @@ sub _djvu2boxes {
     my ( $text, $h ) = @_;
     my @boxes;
     while ( defined $text and $text !~ /\A\s*\z/xsm ) {
-        my @result = extract_bracketed( $text, '()' );
+        my @result = extract_bracketed( $text, '(")' );
+        if ( not defined $result[0] ) {
+            croak "Error parsing brackets in $text";
+        }
         if ( $result[0] =~
             /^\s*[(](\w+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(.*)[)]$/xsm )
         {
