@@ -16,7 +16,7 @@ Readonly my $EMPTY_ARRAY => -1;
 # Glib::ParamSpec->object in Gscan2pdf::Dialog::Scan
 use Glib::Object::Subclass Glib::Object::;
 
-our $VERSION = '1.5.5';
+our $VERSION = '1.6.0';
 
 my $units = qr{(pel|bit|mm|dpi|%|us)}xsm;
 my $EMPTY = q{};
@@ -146,47 +146,6 @@ sub parse_geometry {
     return;
 }
 
-sub synonyms {
-    my ($name) = @_;
-    my @synonyms = (
-        [ scalar(SANE_NAME_PAGE_HEIGHT), 'pageheight' ],
-        [ scalar(SANE_NAME_PAGE_WIDTH),  'pagewidth' ],
-        [ scalar(SANE_NAME_SCAN_TL_X),   'l' ],
-        [ scalar(SANE_NAME_SCAN_TL_Y),   't' ],
-        [ scalar(SANE_NAME_SCAN_BR_X),   'x' ],
-        [ scalar(SANE_NAME_SCAN_BR_Y),   'y' ],
-    );
-    for my $synonym (@synonyms) {
-        given ($name) {
-            when ( @{$synonym} ) {
-                return $synonym;
-            }
-        }
-    }
-    return [$name];
-}
-
-# Note any duplicate options, keeping only the last entry.
-sub prune_duplicates {
-    my ($arrayref) = @_;
-    my %seen;
-
-    my $j = $#{$arrayref};
-    while ( $j > $EMPTY_ARRAY ) {
-        my ($opt) = keys %{ $arrayref->[$j] };
-        my $synonyms = synonyms($opt);
-        for ( @{$synonyms} ) {
-            $seen{$_}++;
-            if ( defined $seen{$_} and $seen{$_} > 1 ) {
-                splice @{$arrayref}, $j, 1;
-                last;
-            }
-        }
-        $j--;
-    }
-    return $arrayref;
-}
-
 sub supports_paper {
     my ( $self, $paper, $tolerance ) = @_;
 
